@@ -17,24 +17,25 @@
 #' \donttest{
 #' S <- cbind(diag(2), c(-1, 1), c(1.1, -1))  # inequality matrix
 #' # S'x >= 0 represents the wedge x1 <= x2 <= 1.1 x1
+#' C <- set_constraints(S=S)  # create a constraints object
 #' # example taken from Pakman and Paninski (2014)
 #' # 1. exact Hamiltonian Monte Carlo (Pakman and Paninski, 2014)
-#' sampler <- create_TMVN_sampler(Q=diag(2), mu=c(4, 4), S=S, method="HMC")
+#' sampler <- create_TMVN_sampler(Q=diag(2), mu=c(4, 4), constraints=C, method="HMC")
 #' sim <- MCMCsim(sampler, n.iter=600, verbose=FALSE)
 #' summary(sim)
 #' plot(as.matrix(sim$x), pch=".")
 #' # 2. exact Hamiltonian Monte Carlo with Laplace momentum (Nishimura et al., 2021)
-#' sampler <- create_TMVN_sampler(Q=diag(2), mu=c(4, 4), S=S, method="HMCZigZag")
+#' sampler <- create_TMVN_sampler(Q=diag(2), mu=c(4, 4), constraints=C, method="HMCZigZag")
 #' sim <- MCMCsim(sampler, n.iter=600, verbose=FALSE)
 #' summary(sim)
 #' plot(as.matrix(sim$x), pch=".")
 #' # 3. Gibbs sampling approach (Rodriguez-Yam et al., 2004)
-#' sampler <- create_TMVN_sampler(Q=diag(2), mu=c(4, 4), S=S, method="Gibbs")
+#' sampler <- create_TMVN_sampler(Q=diag(2), mu=c(4, 4), constraints=C, method="Gibbs")
 #' sim <- MCMCsim(sampler, burnin=500, n.iter=2000, verbose=FALSE)
 #' summary(sim)
 #' plot(as.matrix(sim$x), pch=".")
 #' # 4. soft TMVN approximation (Souris et al., 2018)
-#' sampler <- create_TMVN_sampler(Q=diag(2), mu=c(4, 4), S=S, method="softTMVN")
+#' sampler <- create_TMVN_sampler(Q=diag(2), mu=c(4, 4), constraints=C, method="softTMVN")
 #' sim <- MCMCsim(sampler, n.iter=600, verbose=FALSE)
 #' summary(sim)
 #' plot(as.matrix(sim$x), pch=".")
@@ -51,12 +52,9 @@
 #'  Currently only supported by methods 'direct' and 'HMC'.
 #' @param name name of the TMVN vector parameter.
 #' @param coef.names optional labels for the components of the vector parameter.
-#' @param R equality restriction matrix.
-#' @param r rhs vector for equality constraints \eqn{R'x = r}, where \eqn{R'} denotes the transpose of R.
-#' @param S inequality restriction matrix.
-#' @param s rhs vector for inequality constraints \eqn{S'x >= s}, where \eqn{S'} denotes the transpose of S.
-#' @param lower alternative to \code{s} for two-sided inequality restrictions \eqn{\code{lower} <= S'x <= \code{upper}}.
-#' @param upper alternative to \code{s} for two-sided inequality restrictions \eqn{\code{lower} <= S'x <= \code{upper}}.
+#' @param constraints optional linear equality and/or inequality constraints.
+#'  Use function \code{\link{set_constraints}} to specify the constraint matrices
+#'  and right-hand sides.
 #' @param check.constraints if \code{TRUE} check whether the starting values satisfy all constraints.
 #' @param method sampling method. The options are "direct" for direct sampling from the
 #'  unconstrained or equality constrained multivariate normal (MVN). For inequality constrained
@@ -87,9 +85,9 @@
 #'    and student-t distributions subject to linear inequality constraints.
 #'    Journal of Statistical Theory and Practice 9(4), 712-732.
 #'
-#'  A. Nishimura, Z. Zhang and M.A. Suchard (2021). Hamiltonian zigzag sampler got more momentum
-#'    than its Markovian counterpart: Equivalence of two zigzags under a momentum refreshment limit.
-#'    arXiv:2104.07694.
+#'  A. Nishimura, Z. Zhang and M.A. Suchard (2024). Zigzag path connects two Monte Carlo samplers:
+#'    Hamiltonian counterpart to a piecewise deterministic Markov process.
+#'    Journal of the American Statistical Association, 1-13.
 #'
 #'  A. Pakman and L. Paninski (2014).
 #'    Exact Hamiltonian Monte Carlo for truncated multivariate gaussians.
@@ -103,16 +101,16 @@
 #'    Gaussian Markov Random Fields.
 #'    Chapman & Hall/CRC.
 #'
-#'  A. Souris, A. Bhattacharya and P. Debdeep (2018).
-#'    The Soft Multivariate Truncated Normal Distribution.
-#'    arXiv:1807.09155.
+#'  A. Souris, A. Bhattacharya and P. Debdeep (2019).
+#'    The Soft Multivariate Truncated Normal Distribution with Applications to Bayesian Constrained Estimation.
+#'    arXiv:1807.09155v2.
 #'
 #'  K.A. Valeriano, C.E. Galarza and L.A. Matos (2023).
 #'    Moments and random number generation for the truncated elliptical family of distributions.
 #'    Statistics and Computing 33(1), 1-20.
 create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=update.Q,
                                 name="x", coef.names=NULL,
-                                R=NULL, r=NULL, S=NULL, s=NULL, lower=NULL, upper=NULL,
+                                constraints=NULL,
                                 check.constraints = FALSE,
                                 method=NULL, reduce=NULL,
                                 chol.control=chol_control(), debug=FALSE) {
@@ -137,107 +135,21 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
     names(coef.names) <- name
   }
 
-  if (is.null(R)) {
-    reduce <- FALSE
-    eq <- FALSE
+  if (is.null(constraints)) {
+    eq <- ineq <- FALSE
   } else {
-    if (check.constraints) Rnames <- colnames(R)
-    R <- economizeMatrix(R, vec.as.diag=FALSE, check=TRUE)
-    if (nrow(R) != n || ncol(R) > n) stop("incompatible constraint matrix 'R'")
-    if (is.null(r)) {
-      r <- numeric(ncol(R))
-    } else {
-      if (length(r) != ncol(R)) stop("length of 'r' should equal the number of columns of 'R'")
+    if (!is.environment(constraints)) stop("unexpected constraints object; please use function set_constraints to specify linear constraints")
+    if (constraints[["n"]] != n) stop("constraint size not compatible")
+    eq <- constraints[["eq"]]
+    ineq <- constraints[["ineq"]]
+    if (eq) {
+      R <- constraints[["R"]]
+      r <- constraints[["r"]]
     }
-    # TODO check that R has full column rank
-    if (ncol(R) > 0L) {
-      if (check.constraints) {
-        R0 <- R
-        r0 <- r
-        check_equalities <- function(x, tol=sqrt(.Machine$double.eps)) {
-          tol <- abs(tol)
-          dif <- crossprod_mv(R0, x) - r0
-          viol <- which(abs(dif) > tol)
-          if (length(viol) >= 1L) {
-            warn(length(viol), " equality restriction(s) violated")
-            message("largest discrepancies:")
-            o <- viol[order(abs(dif[viol]), decreasing = TRUE)[1:min(3L, length(viol))]]
-            fdif <- format(dif[o], digits=3L)
-            message(if (is.null(Rnames)) fdif else paste0(fdif, " (", Rnames[o], ")", collapse=", "))
-          }
-        }
-      }
-      eq <- TRUE
-    } else {
-      eq <- FALSE
-    }
-  }
-  if (!eq) rm(R, r)
-
-  if (is.null(S)) {
-    rm(S, s)
-    ineq <- FALSE
-  } else {
-    if (check.constraints) {
-      Snames <- colnames(S)
-      if (is.null(Snames)) Snames <- seq_len(ncol(S))
-    }
-    if (!is.null(lower)) {
-      # use lower and upper bounds, and translate to Sx >= s
-      if (is.null(upper)) stop("lower and upper bound should be specified together")
-      if (!is.null(s)) warn("argument 's' ignored in combination with 'lower' and 'upper'")
-      if (length(lower) != length(upper) || any(lower >= upper)) stop("'lower' and 'upper' incompatible")
-      if (length(lower) != ncol(S)) stop("'S' incompatible with 'lower' and 'upper'")
-      if (any(is.infinite(lower)) || any(is.infinite(upper))) {
-        S <- cbind(S[, is.finite(lower), drop=FALSE], -S[, is.finite(upper), drop=FALSE])
-        s <- c(lower[is.finite(lower)], upper[is.finite(upper)])
-        if (!length(s)) stop("no effective inequalities")
-        if (check.constraints) Snames <- c(paste(Snames[is.finite(lower)], "lower"), paste(Snames[is.finite(upper)], "upper"))
-      } else {
-        S <- cbind(S, -S)
-        s <- c(lower, -upper)
-        if (check.constraints) Snames <- paste(c(Snames, Snames), rep_each(c("lower", "upper"), ncol(S)))
-      }
-    }
-    S <- economizeMatrix(S, vec.as.diag=FALSE, check=TRUE)
-    if (nrow(S) != n) stop("incompatible constraint matrix 'S'")
-    ncS <- ncol(S)
-    if (is.null(s)) {
-      s <- numeric(ncS)
-    } else {
-      s <- as.numeric(s)
-      if (length(s) != ncS) stop("length of 's' should equal the number of columns of 'S'")
-    }
-    if (length(s)) {
-      if (check.constraints) {
-        S0 <- S
-        s0 <- s
-        check_inequalities <- function(x, tol=sqrt(.Machine$double.eps)) {
-          dif <- crossprod_mv(S0, x) - s0
-          viol <- which(dif < -abs(tol))
-          if (length(viol) >= 1L) {
-            warn(length(viol), " inequality restriction(s) violated")
-            message("largest discrepancies:")
-            o <- viol[order(dif[viol])[seq_len(min(3L, length(viol)))]]
-            fdif <- format(dif[o], digits=3L)
-            message(if (is.null(Snames)) fdif else paste0(fdif, " (", Snames[o], ")", collapse=", "))
-          }
-        }
-      }
-      ineq <- TRUE
-    } else {
-      ineq <- FALSE
-    }
-  }
-  rm(lower, upper)
-
-  if (!eq && !ineq) check.constraints <- FALSE
-
-  if (check.constraints) {
-    check_constraints <- function(x, tol=sqrt(.Machine$double.eps)) {
-      tol <- abs(tol)
-      if (eq) check_equalities(x, tol)
-      if (ineq) check_inequalities(x, tol)
+    if (ineq) {
+      S <- constraints[["S"]]
+      s <- constraints[["s"]]
+      ncS <- constraints[["ncS"]]
     }
   }
 
@@ -247,22 +159,22 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
     method <- eval(call(paste0("m_", method)))
   }
   if (ineq) {
-    if (method$name == "direct") stop("method 'direct' cannot be used for inequality constrained sampling")
+    if (method[["name"]] == "direct") stop("method 'direct' cannot be used for inequality constrained sampling")
   } else {
-    if (method$name == "softTMVN") stop("method 'softTMVN' can only be used for inequality constrained sampling")
+    if (method[["name"]] == "softTMVN") stop("method 'softTMVN' can only be used for inequality constrained sampling")
   }
-  
+
   if (is.null(reduce))
-    reduce <- eq && method$name == "Gibbs"
-  else if (eq && method$name == "Gibbs" && !reduce)
+    reduce <- eq && method[["name"]] == "Gibbs"
+  else if (eq && method[["name"]] == "Gibbs" && !reduce)
     stop("for method 'Gibbs' equality restrictions are only supported with 'reduce=TRUE'")
   if (update.Q || update.mu) {
-    if (all(method$name != c("direct", "HMC"))) stop("'update.Q=TRUE' or 'update.mu=TRUE' only supported for methods 'direct' and 'HMC'")
+    if (all(method[["name"]] != c("direct", "HMC"))) stop("'update.Q=TRUE' or 'update.mu=TRUE' only supported for methods 'direct' and 'HMC'")
     if (reduce) stop("'update.Q=TRUE' or 'update.mu=TRUE' not supported in combination with 'reduce=TRUE'")
   }
-  if (reduce || (ineq && method$name == "Gibbs")) name.tr <- paste0(name, "_tr_")
+  if (reduce || (ineq && method[["name"]] == "Gibbs")) name.tr <- paste0(name, "_tr_")
 
-  if (method$name == "direct" && is.null(method$use.cholV)) {
+  if (method[["name"]] == "direct" && is.null(method[["use.cholV"]])) {
     method$use.cholV <- !reduce && !update.Q && n <= 1000L
   }
 
@@ -271,15 +183,15 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
     if (!is.null(Xy)) mu <- build_chol(Q, control=chol.control)$solve(Xy)
 
     # transform to subspace defined by R
-    if (method$name == "softTMVN" || method$name == "HMCZigZag") {
+    if (method[["name"]] == "softTMVN" || method[["name"]] == "HMCZigZag") {
       reduced.frame <- eliminate_equalities(R, r, Q, mu, keep.Q=TRUE, chol.control)
-      Q <- reduced.frame$Q
+      Q <- reduced.frame[["Q"]]
     } else {
       reduced.frame <- eliminate_equalities(R, r, Q, mu, chol.control=chol.control)
       rm(Q)
     }
-    cholQ <- reduced.frame$cholQ
-    n.frame <- reduced.frame$n
+    cholQ <- reduced.frame[["cholQ"]]
+    n.frame <- reduced.frame[["n"]]
     mu <- numeric(n.frame)  # mean is zero in the new frame
 
     if (ineq) {
@@ -290,20 +202,18 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
 
   } else {  # !reduce
 
-    if (isTRUE(method$use.cholV)) {
+    if (isTRUE(method[["use.cholV"]])) {
       tryCatch(
         suppressWarnings({
           V <- economizeMatrix(solve(Q), symmetric=TRUE)
           cholV <- economizeMatrix(chol(V))
         }),
         error = function(err) {
-          # TODO using determinant is not a very good way to check for non-pd
-          d <- determinant(Q, logarithm=FALSE)
-          if (d$sign * d$modulus < .Machine$double.eps) {
-            stop("Non-positive-definite matrix in model component `", name, "`. Consider using 'remove.redundant=TRUE' or increasing the coefficients' prior precision.")
-          } else {
+          if (length(detect_redundancy(Q)))
+            stop("Non-positive-definite matrix in model component `", name,
+                 "`. Consider using 'remove.redundant=TRUE' or increasing the coefficients' prior precision.")
+          else
             stop(err)
-          }
         }
       )
       # remove any cached Cholesky factorizations
@@ -312,42 +222,41 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
       cholQ <- build_chol(Q, control=chol.control)
     }
 
-    if (all(method$name != c("softTMVN", "HMCZigZag"))) rm(Q)
+    if (all(method[["name"]] != c("softTMVN", "HMCZigZag"))) rm(Q)
 
     if (!is.null(Xy)) {
-      mu <- if (isTRUE(method$use.cholV)) V %m*v% Xy else cholQ$solve(Xy)
+      mu <- if (isTRUE(method[["use.cholV"]])) V %m*v% Xy else cholQ$solve(Xy)
     }
 
     if (ineq) {
       # currently tabMatrix disallowed because used as solve rhs below
-      S <- economizeMatrix(S, sparse=if (class(cholQ$cholM)[1L] == "matrix") FALSE else NULL, allow.tabMatrix=FALSE)
+      S <- economizeMatrix(S, sparse=if (cholQ[["type"]] == "matrix") FALSE else NULL, allow.tabMatrix=FALSE)
     }
-    if (method$name == "Gibbs" || method$name == "HMCZigZag") n.frame <- n
+    if (method[["name"]] == "Gibbs" || method[["name"]] == "HMCZigZag") n.frame <- n
 
-    if (eq && any(method$name == c("direct", "HMC"))) {
+    if (eq && any(method[["name"]] == c("direct", "HMC"))) {
       # conditioning by Kriging to deal with equality constraints R'x = r
       # update.Q, dense cholQ --> faster to have dense R (and S) as well
-      R <- economizeMatrix(R, sparse=if (update.Q && class(cholQ$cholM)[1L] == "matrix") FALSE else NULL, allow.tabMatrix=FALSE)
-      if (isTRUE(method$use.cholV))
+      R <- economizeMatrix(R, sparse=if (update.Q && cholQ[["type"]] == "matrix") FALSE else NULL, allow.tabMatrix=FALSE)
+      if (isTRUE(method[["use.cholV"]]))
         projector <- create_projector(V=V, R=R)
       else
         projector <- create_projector(cholQ=cholQ, update.Q=update.Q, R=R)
-      if (all(r == 0)) r <- NULL
     }  # END if (eq)
   }  # END !reduce
-  if (isTRUE(method$use.cholV) && !update.mu) rm(V)
+  if (isTRUE(method[["use.cholV"]]) && !update.mu) rm(V)
   rm(Xy)
 
   self <- environment()
 
 
-  zero.mu <- !update.mu && all(mu == 0)
-  if (any(method$name == c("HMC", "HMCZigZag"))) {
-    if (eq && !reduce && any(r != 0)) zero.mu <- FALSE
+  zero.mu <- !update.mu && allv(mu, 0)
+  if (any(method[["name"]] == c("HMC", "HMCZigZag"))) {
+    if (eq && !reduce && !is.null(r)) zero.mu <- FALSE
   }
   if (zero.mu) mu <- numeric(0L)
 
-  if (method$name == "Gibbs") {
+  if (method[["name"]] == "Gibbs") {
     if (eq) {
       transform <- function(x) cholQ$Ltimes(reduced.frame$transform(x))
       untransform <- function(z) reduced.frame$untransform(cholQ$solve(z, system="Lt"))
@@ -366,28 +275,28 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
 
   # BEGIN draw function
   draw <- if (debug) function(p) {browser()} else function(p) {}
-  if (method$name != "direct" && !(method$name == "Gibbs" && !ineq)) {
-    if (reduce || (method$name == "Gibbs"))
+  if (method[["name"]] != "direct" && !(method[["name"]] == "Gibbs" && !ineq)) {
+    if (reduce || (method[["name"]] == "Gibbs"))
       draw <- add(draw, bquote(x <- p[[.(name.tr)]]))
     else
       draw <- add(draw, bquote(x <- p[[.(name)]]))
   }
 
-  if (method$name == "direct") {
+  if (method[["name"]] == "direct") {
     if (update.Q) draw <- add(draw, quote(cholQ$update(Q, Imult)))
     if (zero.mu)
-      if (method$use.cholV)
+      if (method[["use.cholV"]])
         draw <- add(draw, bquote(x <- drawMVN_cholV(.(n), cholV, scale)))
       else
         draw <- add(draw, quote(x <- drawMVN_cholQ(cholQ, sd=scale)))
     else {
       if (update.mu)
-        if (method$use.cholV)
+        if (method[["use.cholV"]])
           draw <- add(draw, bquote(x <- V %m*v% Xy + drawMVN_cholV(.(n), cholV, scale)))
         else
           draw <- add(draw, quote(x <- drawMVN_cholQ(cholQ, Xy, sd=scale)))
       else
-        if (method$use.cholV)
+        if (method[["use.cholV"]])
           draw <- add(draw, bquote(x <- mu + drawMVN_cholV(.(n), cholV, scale)))
         else
           draw <- add(draw, quote(x <- mu + drawMVN_cholQ(cholQ, sd=scale)))
@@ -399,11 +308,11 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
     }
   }  # END direct
 
-  if (method$name == "Gibbs") {
+  if (method[["name"]] == "Gibbs") {
     if (ineq) {
       # transform to zero mean unit covariance matrix frame
       # inequalities U'v >= u
-      u <- if (zero.mu) s else s - crossprod_mv(S, mu)
+      u <- if (zero.mu) s else if (is.null(s)) -crossprod_mv(S, mu) else s - crossprod_mv(S, mu)
       # drop very small numbers in U as they give problems in the Gibbs sampler
       # and use transpose for faster col access in sparse case
       Ut <- economizeMatrix(
@@ -411,11 +320,14 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
         vec.diag=FALSE, allow.tabMatrix=FALSE
       )
       if (all(class(Ut)[1L] != c("matrix", "dgCMatrix"))) Ut <- as(as(Ut, "CsparseMatrix"), "generalMatrix")
-      
-      eps <- method$eps
+
+      eps <- method[["eps"]]
+      if (is.null(u))
+        draw <- add(draw, quote(ustar <- -(Ut %m*v% x)))
+      else
+        draw <- add(draw, quote(ustar <- u - Ut %m*v% x))
       # draw from truncated univariate normal full conditionals
-      draw <- add(draw, quote(ustar <- u - Ut %m*v% x))
-      if (method$slice) {
+      if (method[["slice"]]) {
         if (is.matrix(Ut))
           draw <- add(draw, quote(x <- Crtmvn_slice_Gibbs_dense(x, Ut, ustar, eps)))
         else
@@ -431,11 +343,11 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
     }
   }  # END Gibbs
 
-  if (method$name == "softTMVN") {
-    useV <- method$useV
-    if (!is.null(method$CG) && eq) stop("conjugate gradients with equality constraints is not supported")
-    if (method$PG.approx) {
-      mPG <- as.integer(method$PG.approx.m)
+  if (method[["name"]] == "softTMVN") {
+    useV <- method[["useV"]]
+    if (!is.null(method[["CG"]]) && eq) stop("conjugate gradients with equality constraints is not supported")
+    if (method[["PG.approx"]]) {
+      mPG <- as.integer(method[["PG.approx.m"]])
       if (all(length(mPG) != c(1L, n))) stop("invalid value for option 'PG.approx.m'")
       rPolyaGamma <- function(b, c) CrPGapprox(ncS, b, c, mPG)
     } else {
@@ -469,16 +381,23 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
         force.sparse = eq && inherits(R, "CsparseMatrix")
       )
       ch <- build_chol(matsum(crossprod_sym(St, runif(ncS, 0.9, 1.1))), control=chol.control)
-      if (eq && !reduce)
+      if (eq && !reduce) {
+        # conditioning by Kriging to deal with equality constraints R'x = r
+        # update.Q, dense cholQ --> faster to have dense R (and S) as well
+        R <- economizeMatrix(R, sparse=if (update.Q && cholQ[["type"]] == "matrix") FALSE else NULL, allow.tabMatrix=FALSE)
         projector <- create_projector(cholQ=ch, update.Q=TRUE, R=R)
+      }
     }
 
-    sharpness <- method$sharpness
+    sharpness <- method[["sharpness"]]
+    if (ncS > 1L && length(sharpness) == 1L) sharpness <- rep.int(sharpness, ncS)
     sharpness_squared <- sharpness^2
-    sharpness_inv <- 1/sharpness
     # 1. draw Polya-Gamma mixture precisions
+    if (is.null(s))
+      draw <- add(draw, quote(discr <- crossprod_mv(S, x)))
+    else
+      draw <- add(draw, quote(discr <- crossprod_mv(S, x) - s))
     draw <- draw |>
-      add(quote(discr <- crossprod_mv(S, x) - s)) |>
       add(quote(omega <- rPolyaGamma(1, sharpness * discr))) |>
       # 2. draw vector of coefficients
       add(quote(omega.tilde <- sharpness_squared * omega))
@@ -486,27 +405,36 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
       draw <- draw |>
         add(quote(attr(D.omega.tilde.inv, "x") <- 1 / omega.tilde)) |>
         add(quote(XX_V <- matsum(D.omega.tilde.inv))) |>
-        add(quote(ch$update(XX_V))) |>
+        add(quote(ch$update(XX_V)))
+      if (is.null(s))
+        add(quote(alpha <- 0.5/omega))
+      else
         add(quote(alpha <- sharpness * s + 0.5/omega))
       if (zero.mu)
         draw <- add(draw, quote(y1 <- drawMVN_cholQ(cholQ)))
       else
         draw <- add(draw, quote(y1 <- mu + drawMVN_cholQ(cholQ)))
-      draw <- add(draw, bquote(y2 <- Crnorm(.(length(s))) * sqrt(1/omega)))
+      draw <- add(draw, bquote(y2 <- Crnorm(.(length(s))) * invsqrt(omega)))
+      sharpness_inv <- 1/sharpness
       draw <- add(draw, quote(x <- y1 + VS %m*v% ch$solve(sharpness_inv * (alpha - y2) - crossprod_mv(S, y1))))
       if (eq && !reduce) {
-        draw <- draw |>
-          add(quote(cholRVxR$update(matsum_RVxR(M1=crossprod_sym2(SVR, ch$solve(SVR)), w1=-1)))) |>
-          add(quote(temp <- V %m*v% (R %m*v% cholRVxR$solve(r - crossprod_mv(R, x))))) |>
-          add(quote(x <- x + temp - V %m*v% (S %m*v% ch$solve(crossprod_mv(S, temp)))))
+        draw <- add(draw, quote(cholRVxR$update(matsum_RVxR(M1=crossprod_sym2(SVR, ch$solve(SVR)), w1=-1))))
+        if (is.null(r))
+          draw <- add(draw, quote(temp <- V %m*v% (R %m*v% cholRVxR$solve(-crossprod_mv(R, x)))))
+        else
+          draw <- add(draw, quote(temp <- V %m*v% (R %m*v% cholRVxR$solve(r - crossprod_mv(R, x)))))
+        draw <- add(draw, quote(x <- x + temp - V %m*v% (S %m*v% ch$solve(crossprod_mv(S, temp)))))
       }
     } else {
-      draw <- add(draw, quote(alpha <- 0.5 * sharpness + s * omega.tilde))
+      if (is.null(s))
+        draw <- add(draw, quote(alpha <- 0.5 * sharpness))
+      else
+        draw <- add(draw, quote(alpha <- 0.5 * sharpness + s * omega.tilde))
       if (zero.mu)
         draw <- add(draw, quote(Xy <- crossprod_mv(St, alpha)))
       else
         draw <- add(draw, quote(Xy <- crossprod_mv(St, alpha) + Qmu))
-      if (is.null(method$CG)) {
+      if (is.null(method[["CG"]])) {
         # Cholesky
         draw <- draw |>
           add(quote(XX <- crossprod_sym(St, omega.tilde))) |>
@@ -522,19 +450,19 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
         # conjugate gradients
         draw <- add(draw, bquote(u <- Xy + crossprod_mv(St, sqrt(omega.tilde) * Crnorm(.(ncS))) + cholQ$Ltimes(Crnorm(.(n)), transpose=FALSE)))
         A_times <- function(x, omega.tilde) crossprod_mv(St, omega.tilde * (St %m*v% x)) + Q %m*v% x
-        dQinv <- sqrt(1/diag(Q))  # seems better as a preconditioner than 1/diag(Q)
+        dQinv <- invsqrt(diag(Q))  # seems better as a preconditioner than 1/diag(Q)
         M_solve <- function(x, omega.tilde) dQinv * x
-        max.it <- method$CG$max.it
-        stop.criterion <- method$CG$stop.criterion
-        verbose <- method$CG$verbose
+        max.it <- method$CG[["max.it"]]
+        stop.criterion <- method$CG[["stop.criterion"]]
+        verbose <- method$CG[["verbose"]]
         draw <- add(draw, quote(x <- CG(u, self, x, max.it=max.it,
-            e=stop.criterion, verbose=verbose, omega.tilde=omega.tilde)
+          e=stop.criterion, verbose=verbose, omega.tilde=omega.tilde)
         ))
       }
     }
   }  # END softTMVN
 
-  if (method$name == "HMC") {
+  if (method[["name"]] == "HMC") {
     # Hamiltonian H = 1/2 x'Mx - g'x + 1/2 p'M^{-1}p
     # precision matrix M, covariance matrix M^{-1}, mu=M^{-1}g must obey equality restrictions
     # Hamilton's equations: dx/dt = M^{-1}p, p = M dx/dt
@@ -549,10 +477,10 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
         VS <- economizeMatrix(cholQ$solve(S), allow.tabMatrix=FALSE)
       } else if (!update.Q) {
         # define VS as Q^-1 S = V S; alternatively transform after projection
-        VS <- economizeMatrix(cholQ$solve(S), sparse=if (is.matrix(cholQ$cholM)) FALSE else NULL, allow.tabMatrix=FALSE)
+        VS <- economizeMatrix(cholQ$solve(S), sparse=if (cholQ[["type"]] == "matrix") FALSE else NULL, allow.tabMatrix=FALSE)
         if (eq) {
           # project inequality matrix on equality constraint surface, use Q as a metric
-          VS <- economizeMatrix(projector$project(VS, cholQ), sparse=if (is.matrix(cholQ$cholM)) FALSE else NULL, allow.tabMatrix=FALSE)
+          VS <- economizeMatrix(projector$project(VS, cholQ), sparse=if (cholQ[["type"]] == "matrix") FALSE else NULL, allow.tabMatrix=FALSE)
         }
       } else {
         VS <- NULL
@@ -563,13 +491,15 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
         simplified <- FALSE
       } else {
         refl.fac <- 2 / colSums(S * VS)  # 2 / vector of normal' Q normal for all inequalities
-        s.adj <- if (zero.mu) s else s - crossprod_mv(S, mu)  # if positive, mu violates inequalities
-        abs.s.adj <- abs(s.adj)
+        if (is.null(s))
+          s.adj <- if (zero.mu) numeric(ncS) else -crossprod_mv(S, mu)
+        else
+          s.adj <- if (zero.mu) s else s - crossprod_mv(S, mu)  # if positive, mu violates inequalities
         simplified <- !eq && identical(VS, S)  # simplified=TRUE is the case described in Pakman and Paninski: identity Q and no equality constraints
         if (simplified) VS <- NULL
       }
-      max.events <- method$max.events
-      diagnostic <- method$diagnostic
+      max.events <- method[["max.events"]]
+      diagnostic <- method[["diagnostic"]]
       if (diagnostic) {
         # set up a vector of wall bounce counts
         bounces <- setNames(integer(ncS), if (is.null(colnames(S))) seq_len(ncS) else colnames(S))
@@ -593,9 +523,10 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
       draw <- add(draw, quote(mu <- cholQ$solve(Xy)))
       if (eq)
         draw <- add(draw, quote(mu <- projector$project(mu, cholQ, r)))
-      draw <- draw |>
-        add(quote(s.adj <- s - crossprod_mv(S, mu))) |>  # if positive, mu violates inequalities
-        add(quote(abs.s.adj <- abs(s.adj)))
+      if (is.null(s))
+        draw <- add(draw, quote(s.adj <- -crossprod_mv(S, mu)))
+      else
+        draw <- add(draw, quote(s.adj <- s - crossprod_mv(S, mu)))  # if positive, mu violates inequalities
     }
 
     # 1. draw p from N(0, M^{-1}); instead draw v=dx/dt from N(0, M)
@@ -614,7 +545,7 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
     if (ineq) {
       if (diagnostic) {
         draw <- draw |>
-          add(quote(viol <- crossprod_mv(S, x) < s)) |>
+          add(bquote(viol <- crossprod_mv(S, x) < .(if (is.null(s)) 0 else quote(s)))) |>
           add(quote(if (any(viol)) cat("\nviolated constraints:", names(bounces)[viol])))
       }
       # NB bounces is updated by reference
@@ -632,7 +563,7 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
     }
   }  # END HMC
 
-  if (method$name == "HMCZigZag") {
+  if (method[["name"]] == "HMCZigZag") {
     # TODO: update.Q and update.mu cases
 
     base_which <- base::which  # faster, in case Matrix::which is loaded
@@ -643,7 +574,7 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
       iQ <- xQ <- list()
       for (j in seq_len(ncol(Q))) {
         temp <- Q[, j, drop=TRUE]
-        iQ[[j]] <- base_which(temp != 0)
+        iQ[[j]] <- whichv(temp, 0, invert=TRUE)
         xQ[[j]] <- temp[temp != 0]
       }
     }
@@ -653,7 +584,7 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
       if (class(S)[1L] == "dgCMatrix") {
         # store nonzero indices for each column
         inds <- list()
-        len <- diff(S@p)
+        len <- diff.default(S@p)
         for (j in seq_len(ncS)) {
           inds[[j]] <- S@i[(S@p[j] + 1L):(S@p[j] + len[j])] + 1L
         }
@@ -662,28 +593,27 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
 
     if (eq && !reduce) {
       # TODO can we use the approximate precision matrix form here using matrix-inversion lemma?
-      #      then we would nowhere need to factorize Q
+      #      then we would nowhere need to factorise Q
       # project mean mu on eq constraint surface
       VR <- cholQ$solve(R)
-      if (!zero.mu) {
-        VR.RVRinv <- economizeMatrix(VR %*% solve(crossprod_sym2(R, VR)), drop.zeros=TRUE)
-        mu <- mu + VR.RVRinv %m*v% (r - crossprod_mv(R, mu))
-        rm(VR.RVRinv)
-      }
-      if (is.null(method$prec.eq))
+      if (!zero.mu)
+        mu <- mu + VR %m*v% build_chol(crossprod_sym2(R, VR))$solve(
+          if (is.null(r)) -crossprod_mv(R, mu) else r - crossprod_mv(R, mu)
+        )
+      if (is.null(method[["prec.eq"]]))
         prec.eq <- 100 * diag(solve(crossprod_sym2(R, VR)))
       else
-        prec.eq <- method$prec.eq
+        prec.eq <- method[["prec.eq"]]
       rm(VR)
     }
 
-    adapt <- method$adapt
-    rate <- method$rate
+    adapt <- method[["adapt"]]
+    rate <- 1/method[["scale"]]
     # NB if (reduce) then length of rate should be either 1 or the dimension of the reduced frame
     if (length(rate) == 1L) rate <- rep.int(rate, n.frame)
-    if (length(rate) != n.frame) stop("Laplace distribution rate parameter has wrong length")
-    unit.rate <- all(rate == 1) && !adapt
-    diagnostic <- method$diagnostic
+    if (length(rate) != n.frame) stop("Laplace distribution scale parameter has wrong length")
+    unit.rate <- allv(rate, 1) && !adapt
+    diagnostic <- method[["diagnostic"]]
     if (diagnostic) {
       gradbounces <- setNames(integer(n), seq_len(n))
       display.n <- seq_len(min(10L, n))  # number of counts to display
@@ -703,7 +633,7 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
       add(quote(Tleft <- T0))
     if (diagnostic && ineq) {
       draw <- draw |>
-        add(quote(viol <- crossprod_mv(S, x) < s)) |>
+        add(bquote(viol <- crossprod_mv(S, x) < .(if (is.null(s)) 0 else quote(s)))) |>
         add(quote(if (any(viol)) cat("\nviolated inequality constraints:", names(bounces)[viol])))  # print violated constraints
     }
     # draw pM from Laplace distribution, with parameters w; use pM for momentum to distinguish from state p
@@ -794,20 +724,16 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
         # but for numerical robustness (important!!) set it to 0 below
         # update Qx
         if (!eq) Qx <- Qx + dt.1st * Qv
-        # all.equal(Qx, crossprod_mv(Q, x - mu))
         # update v
         if (gradient.event) {
           pM[istar] <- 0  # for numerical robustness (important!)
           v[istar] <- -v[istar]
           if (!eq || reduce) {
             # update Qv (NB v[istar] has already changed sign)
-            #Qv <- Qv + 2 * v[istar] * get_col(Q, istar)
-            if (class(Q)[1L] == "matrix") {
+            if (class(Q)[1L] == "matrix")
               Qv <- Qv + 2 * v[istar] * Q[, istar, drop=TRUE]
-            } else {
+            else
               Qv[iQ[[istar]]] <- Qv[iQ[[istar]]] + 2 * v[istar] * xQ[[istar]]
-            }
-            # all.equal(Qv, crossprod_mv(Q, v))
           }
           if (diagnostic) {
             gradbounces[istar] <- gradbounces[istar] + 1L
@@ -865,8 +791,8 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
     }
   }  # END HMCZigZag
 
-  if (reduce || method$name == "Gibbs") {
-    if (method$name != "direct" && !(method$name == "Gibbs" && !ineq))
+  if (reduce || method[["name"]] == "Gibbs") {
+    if (method[["name"]] != "direct" && !(method[["name"]] == "Gibbs" && !ineq))
       draw <- add(draw, bquote(p[[.(name.tr)]] <- x))
     draw <- add(draw, bquote(p[[.(name)]] <- untransform(x)))
   } else {
@@ -885,7 +811,7 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
   }
   # END draw function
 
-  if (ineq && method$name == "Gibbs") {
+  if (ineq && method[["name"]] == "Gibbs") {
     start <- function(p=list(), scale=1) {
       if (!is.null(p[[name.tr]])) {
         if (length(p[[name.tr]]) != n.frame) stop("wrong length of start value for '", name.tr, "'")
@@ -907,17 +833,17 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
           epsilon <- scale * rexp(ncS)
           res <- lintools::sparse_project(x=x0, A, b=-(u + epsilon), neq=0L, base=0L, sorted=FALSE)
           scale <- 0.3 * scale
-          if (anyNA(res$x)) {
+          if (anyNA(res[["x"]])) {
             x0 <- Crnorm(n.frame, sd=scale)
           } else {
-            if (all(Ut %m*v% res$x >= u)) break
-            x0 <- res$x + Crnorm(n.frame, sd=scale)
+            if (all(Ut %m*v% res[["x"]] >= u)) break
+            x0 <- res[["x"]] + Crnorm(n.frame, sd=scale)
           }
         }
-        p[[name.tr]] <- res$x
+        p[[name.tr]] <- res[["x"]]
         p[[name]] <- untransform(p[[name.tr]])
       }
-      if (check.constraints) check_constraints(p[[name]])
+      if (check.constraints) constraints$check(p[[name]])
       p
     }
   } else {
@@ -936,12 +862,14 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
     } else {
       start <- function(p=list(), scale=1) {
         if (!is.null(p[[name]])) {
+          if (length(p[[name]]) != n) stop("wrong length of start value for '", name, "'")
           p[[name]] <- as.numeric(p[[name]])
+          if (anyNA(p[[name]])) stop("missing(s) in start values")
           return(p)
         }
       }
     }
-    if (isTRUE(method$use.cholV)) {
+    if (isTRUE(method[["use.cholV"]])) {
       if (zero.mu) {
         start <- add(start, bquote(x <- drawMVN_cholV(.(n), cholV, scale)))
       } else {
@@ -955,16 +883,18 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
       }
     }
     if (eq && !reduce) {
-      if (method$name == "softTMVN") {
+      if (method[["name"]] == "softTMVN") {
         if (useV) {
-          start <- start |>
-            add(quote(cholRVxR$update(matsum_RVxR(M1=crossprod_sym2(SVR, ch$solve(SVR)), w1=-1)))) |>
-            add(quote(temp <- V %m*v% (R %m*v% cholRVxR$solve(r - crossprod_mv(R, x))))) |>
-            add(bquote(p[[.(name)]] <- x + temp - V %m*v% (S %m*v% ch$solve(crossprod_mv(S, temp)))))
+          start <- add(start, quote(cholRVxR$update(matsum_RVxR(M1=crossprod_sym2(SVR, ch$solve(SVR)), w1=-1))))
+          if (is.null(r))
+            start <- add(quote(temp <- V %m*v% (R %m*v% cholRVxR$solve(-crossprod_mv(R, x)))))
+          else
+            start <- add(quote(temp <- V %m*v% (R %m*v% cholRVxR$solve(r - crossprod_mv(R, x)))))
+          add(bquote(p[[.(name)]] <- x + temp - V %m*v% (S %m*v% ch$solve(crossprod_mv(S, temp)))))
         } else
           start <- add(start, bquote(p[[.(name)]] <- projector$project(x, ch, r)))
       } else {
-        if (method$name == "HMCZigZag")
+        if (method[["name"]] == "HMCZigZag")
           start <- add(start, bquote(p[[.(name)]] <- x))  # maybe also create and use projector?
         else
           start <- add(start, bquote(p[[.(name)]] <- projector$project(x, cholQ, r)))
@@ -978,7 +908,7 @@ create_TMVN_sampler <- function(Q, mu=NULL, Xy=NULL, update.Q=FALSE, update.mu=u
         start <- add(start, bquote(p[[.(name)]] <- x))
       }
     }
-    if (check.constraints) start <- add(start, bquote(check_constraints(p[[.(name)]])))
+    if (check.constraints) start <- add(start, bquote(constraints$check(p[[.(name)]])))
     start <- add(start, quote(p))
   }
 
