@@ -32,13 +32,14 @@
 #'
 #' # fit Ybarra-Lohr model
 #' sampler <- create_sampler(
-#'   y ~ 1 + mec(~ 0 + W, V=C) + gen(factor=~local_),
-#'   Q0=1/psi, sigma.fixed=TRUE, linpred="fitted"
+#'   y ~ 1 + mec(~ 0 + W, V=C, name="ME") + gen(factor=~local_),
+#'   family=f_gaussian(var.prior=pr_fixed(1), var.vec=~psi),
+#'   linpred="fitted"
 #' )
 #' sim <- MCMCsim(sampler, n.iter=800, n.chain=2, store.all=TRUE, verbose=FALSE)
 #' (summ <- summary(sim))
 #' plot(X, W, xlab="true X", ylab="inferred X")
-#' points(X, summ$mec2_X[, "Mean"], col="green")
+#' points(X, summ$ME_X[, "Mean"], col="green")
 #' abline(0, 1, col="red")
 #' legend("topleft", legend=c("prior mean", "posterior mean"), col=c("black", "green"), pch=c(1,1))
 #' }
@@ -74,16 +75,10 @@
 #'  It can be specified as a scalar value or as a numeric vector of
 #'  appropriate length. DEPRECATED, please use argument \code{prior}
 #'  instead, i.e. \code{prior = pr_normal(mean = b0.value, precision = Q0.value)}.
-#' @param R optional constraint matrix for equality restrictions R'x = r where
-#'  \code{x} is the vector of regression effects.
-#' @param r right hand side for the equality constraints.
-#' @param S optional constraint matrix for inequality constraints S'x >= s where
-#'  x is the vector of regression effects.
-#' @param s right hand side for the inequality constraints.
-#' @param lower as an alternative to \code{s}, \code{lower} and \code{upper} may be specified
-#'  for two-sided constraints lower <= S'x <= upper.
-#' @param upper as an alternative to \code{s}, \code{lower} and \code{upper} may be specified
-#'  for two-sided constraints lower <= S'x <= upper.
+#' @param constraints optional linear equality and/or inequality constraints
+#'  imposed on the vector of regression coefficients. Use function
+#'  \code{\link{set_constraints}} to specify the constraint matrices and
+#'  right-hand sides.
 #' @param name the name of the model component. This name is used in the output of the
 #'  MCMC simulation function \code{\link{MCMCsim}}. By default the name will be 'reg'
 #'  with the number of the model term attached.
@@ -102,7 +97,7 @@
 #'    Scandinavian Journal of Statistics 42(2), 518-529.
 mec <- function(formula = ~ 1, sparse=NULL, X=NULL, V=NULL,
                 prior=NULL, Q0=NULL, b0=NULL,
-                R=NULL, r=NULL, S=NULL, s=NULL, lower=NULL, upper=NULL,
+                constraints=NULL,
                 name="", debug=FALSE) {
 
   e <- sys.frame(-2L)
@@ -113,12 +108,12 @@ mec <- function(formula = ~ 1, sparse=NULL, X=NULL, V=NULL,
   if (is.null(X)) {
     # TODO: warn if formula contains explicit intercept or categorical terms
     vs <- as.list(attr(terms(formula), "variables")[-1L])
-    if (all(lengths(vs) == 3L)) {
-      if (!all(b_apply(vs, function(x) identical(x[[1L]], as.name("|"))))) stop("invalid 'formula' argument")
+    if (allv(lengths(vs), 3L)) {
+      if (!all(b_apply(vs, \(x) identical(x[[1L]], as.name("|"))))) stop("invalid 'formula' argument")
       V.in.formula <- TRUE
-      formula.X <- as.formula(paste0("~ 0 + ", paste0(s_apply(vs, function(x) deparse(x[[2L]])), collapse=" + ")), env=environment(formula))
+      formula.X <- as.formula(paste0("~ 0 + ", paste0(s_apply(vs, \(x) deparse(x[[2L]])), collapse=" + ")), env=environment(formula))
       if (grepl("|", as.character(formula.X)[2L], fixed=TRUE)) stop("invalid 'formula'")  # maybe '()' forgotten?
-      formula.V <- as.formula(paste0("~ 0 + ", paste0(s_apply(vs, function(x) deparse(x[[3L]])), collapse=" + ")), env=environment(formula))
+      formula.V <- as.formula(paste0("~ 0 + ", paste0(s_apply(vs, \(x) deparse(x[[3L]])), collapse=" + ")), env=environment(formula))
       X <- model_matrix(formula.X, e[["data"]], sparse=sparse)
       V <- model_matrix(formula.V, e[["data"]], sparse=sparse)
     } else {
@@ -131,14 +126,14 @@ mec <- function(formula = ~ 1, sparse=NULL, X=NULL, V=NULL,
       }
     }
   } else {
-    if (is.null(colnames(X))) colnames(X) <- seq_len(ncol(X))
+    if (is.null(dimnames(X)[[2L]])) colnames(X) <- seq_len(ncol(X))
     V.in.formula <- FALSE
   }
   if (nrow(X) != e[["n"]]) stop("design matrix with incompatible number of rows")
-  e$coef.names[[name]] <- colnames(X)
+  e$coef.names[[name]] <- dimnames(X)[[2L]]
   X <- economizeMatrix(X, sparse=sparse, strip.names=TRUE, check=TRUE)
   q <- ncol(X)
-  in_block <- any(name == unlst(e$block))
+  in_block <- any(name == unlst(e[["block"]]))
 
   if (!V.in.formula) {
     if (is.null(V)) stop("no measurement error variances supplied in mec component")
@@ -152,9 +147,9 @@ mec <- function(formula = ~ 1, sparse=NULL, X=NULL, V=NULL,
   # determine units with measurement error
   V <- unname(V)
   if (any(V < 0)) stop("negative measurement error variance(s)")
-  i.me <- which(apply(V, 1L, function(x) all(x > 0)))  # TODO add tolerance + allow list of matrix
+  i.me <- which(dapply(V, \(x) all(x > 0), MARGIN=1L))  # TODO add tolerance + allow list of matrix
   nme <- length(i.me)
-  if (nme == e[["n"]]) i.me <- 1:e[["n"]]  # use ALTREP (?)
+  if (nme == e[["n"]]) i.me <- seq_len(e[["n"]])
   if (q == 1L) {
     QME <- 1 / V[i.me, ]
     rm(V)
@@ -162,10 +157,10 @@ mec <- function(formula = ~ 1, sparse=NULL, X=NULL, V=NULL,
     V <- V[i.me, ]
   }
 
-  if (e$Q0.type == "unit") {
+  if (e[["Q0.type"]] == "unit") {
     if (q == 1L) v0 <- 1 else v0 <- rep.int(1, nme)
   } else {
-    if (e$Q0.type == "symm") stop("not supported: measurement error component and non-diagonal sampling variance")
+    if (e[["Q0.type"]] == "symm") stop("not supported: measurement error component and non-diagonal sampling variance")
     v0 <- 1/diag(e[["Q0"]])[i.me]
   }
 
@@ -175,23 +170,24 @@ mec <- function(formula = ~ 1, sparse=NULL, X=NULL, V=NULL,
   } else {
     if (is.null(prior)) prior <- pr_normal(mean=0, precision=0)
   }
-  if (prior$type != "normal") stop("only a normal prior is currently supported for 'mec' model component effects")
+  if (prior[["type"]] != "normal") stop("only a normal prior is currently supported for 'mec' model component effects")
   prior$init(q, e$coef.names[[name]], sparse=if (in_block) TRUE else NULL, sigma=!e$sigma.fixed)  # mc_block uses x-slot of precision matrix
-  informative.prior <- prior$informative
-  Q0 <- prior$precision
-  zero.mean <- !informative.prior || all(prior$mean == 0)
+  informative.prior <- prior[["informative"]]
+  Q0 <- prior[["precision"]]
+  zero.mean <- !informative.prior || allv(prior[["mean"]], 0)
   if (zero.mean) {
     prior$mean <- 0
     Q0b0 <- 0
   } else {
-    if (length(prior$mean) == 1L)
-      Q0b0 <- Q0 %m*v% rep.int(prior$mean, q)
+    if (length(prior[["mean"]]) == 1L)
+      Q0b0 <- Q0 %m*v% rep.int(prior[["mean"]], q)
     else
-      Q0b0 <- Q0 %m*v% prior$mean
+      Q0b0 <- Q0 %m*v% prior[["mean"]]
   }
-  # TODO TMVN prior
 
-  if (!zero.mean && e$compute.weights) stop("weights cannot be computed if some coefficients have non-zero prior means")
+  is.proper <- informative.prior
+
+  if (!zero.mean && e[["compute.weights"]]) stop("weights cannot be computed if some coefficients have non-zero prior means")
 
   name_X <- paste0(name, "_X")
   lp <- function(p) p[[name_X]] %m*v% p[[name]]
@@ -235,7 +231,7 @@ mec <- function(formula = ~ 1, sparse=NULL, X=NULL, V=NULL,
         linpred <- function(p) (Xnew + SEnew * Crnorm(length(SEnew))) %m*v% p[[name]]
         linpred_update <- function(x, plus=TRUE, p) mv_update(x, plus, Xnew + SEnew * Crnorm(length(SEnew)), p[[name]])
       } else {
-        i.me.new <- which(apply(SEnew, 1L, function(x) all(x > 0)))  # TODO add tolerance + allow list of matrix
+        i.me.new <- which(dapply(SEnew, \(x) all(x > 0), MARGIN=1L))  # TODO add tolerance + allow list of matrix
         if (length(i.me.new) == nrow(newdata)) i.me.new <- seq_len(nrow(newdata))
         SEnew <- SEnew[i.me.new, ]
         linpred <- function(p) {
@@ -256,27 +252,27 @@ mec <- function(formula = ~ 1, sparse=NULL, X=NULL, V=NULL,
 
   rprior <- function(p) {}
   # X, QME are assumed to be part of the prior information
-  if (nme == e[["n"]])
-    rprior <- add(rprior, bquote(p[[.(name_X)]] <- X + sqrt(.(if (q == 1L) quote(1/QME) else quote(V))) * Crnorm(.(length(QME)))))
-  else {
+  if (nme == e[["n"]]) {
+    rprior <- add(rprior, bquote(p[[.(name_X)]] <- X + sqrt(.(if (q == 1L) quote(1/QME) else quote(V))) * Crnorm(.(nme*q))))
+  } else {
     rprior <- rprior |>
       add(bquote(p[[.(name_X)]] <- X)) |>
-      add(bquote(p[[.(name_X)]][i.me, ] <- sqrt(.(if (q == 1L) quote(1/QME) else quote(V))) * Crnorm(.(length(QME)))))
+      add(bquote(p[[.(name_X)]][i.me, ] <- X[i.me, ] + sqrt(.(if (q == 1L) quote(1/QME) else quote(V))) * Crnorm(.(nme*q))))
   }
   rprior <- rprior |>
     add(bquote(p[[.(name)]] <- prior$rprior(p))) |>
     add(quote(p))
 
 
-  if (!e$prior.only) {
+  if (!e[["prior.only"]]) {
 
     if (q > 1L) base_tcrossprod <- base::tcrossprod
 
     draw <- if (debug) function(p) {browser()} else function(p) {}
-    if (e$single.block && length(e$mod) == 1L) {
+    if (e[["single.block"]] && length(e[["mod"]]) == 1L) {
       draw <- add(draw, quote(p$e_ <- e$y_eff()))
     } else {
-      if (e$e.is.res)
+      if (e[["e.is.res"]])
         draw <- add(draw, bquote(mv_update(p[["e_"]], plus=TRUE, p[[.(name_X)]], p[[.(name)]])))
       else
         draw <- add(draw, bquote(mv_update(p[["e_"]], plus=FALSE, p[[.(name_X)]], p[[.(name)]])))
@@ -284,7 +280,7 @@ mec <- function(formula = ~ 1, sparse=NULL, X=NULL, V=NULL,
 
     # 1. draw covariates p[[name_X]]
     # for now we assume that X is dense
-    draw <- add(draw, bquote(scale <- .(if (e$sigma.fixed) quote(v0) else quote(v0 * p[["sigma_"]]^2))))
+    draw <- add(draw, bquote(scale <- .(if (e[["sigma.fixed"]]) quote(v0) else quote(v0 * p[["sigma_"]]^2))))
     if (q == 1L) {
       draw <- add(draw, bquote(Vscaled <- 1 / (QME * scale + p[[.(name)]]^2)))
       if (nme == e[["n"]]) {
@@ -309,14 +305,14 @@ mec <- function(formula = ~ 1, sparse=NULL, X=NULL, V=NULL,
 
     if (!in_block) {
       # 2. draw coefficients p[[name]]
-      if (all(Q0b0 == 0))
+      if (allv(Q0b0, 0))
         draw <- add(draw, bquote(Xy <- crossprod_mv(p[[.(name_X)]], e$Q_e(p))))
       else
         draw <- add(draw, bquote(Xy <- crossprod_mv(p[[.(name_X)]], e$Q_e(p)) + Q0b0))
 
       # TODO modeled.Q case: use p[["Q_"]]
       #if (e$modeled.Q) {  # in this case both XX and Xy must be updated in each iteration
-      if (e$Q0.type == "symm") {
+      if (e[["Q0.type"]] == "symm") {
         # TODO this case not (yet) supported!
         draw <- add(draw, bquote(XX <- crossprod_sym(p[[.(name_X)]], p[["QM_"]])))
       } else {
@@ -333,20 +329,20 @@ mec <- function(formula = ~ 1, sparse=NULL, X=NULL, V=NULL,
         mat_sum <- make_mat_sum(M0=Q0, M1=crossprod_sym(X, crossprod_sym(Cdiag(runif(e[["n"]], 0.9, 1.1)), e[["Q0"]])))
         MVNsampler <- create_TMVN_sampler(
           Q=mat_sum(crossprod_sym(X, crossprod_sym(Cdiag(runif(e[["n"]], 0.9, 1.1)), e[["Q0"]]))),
-          update.Q=TRUE, name=name, R=R, r=r, S=S, s=s, lower=lower, upper=upper
+          update.Q=TRUE, name=name, constraints=constraints
         )
         draw <- add(draw, bquote(p[[.(name)]] <- MVNsampler$draw(p, .(if (e$sigma.fixed) 1 else quote(p[["sigma_"]])), Q=mat_sum(XX), Xy=Xy)[[.(name)]]))
       } else {
         MVNsampler <- create_TMVN_sampler(
           Q=crossprod_sym(X, crossprod_sym(Cdiag(runif(e[["n"]], 0.9, 1.1)), e[["Q0"]])),
-          update.Q=TRUE, name=name, R=R, r=r, S=S, s=s, lower=lower, upper=upper
+          update.Q=TRUE, name=name, constraints=constraints
         )
         draw <- add(draw, bquote(p[[.(name)]] <- MVNsampler$draw(p, .(if (e$sigma.fixed) 1 else quote(p[["sigma_"]])), Q=XX, Xy=Xy)[[.(name)]]))
       }
     }  # END if (!in_block)
 
     # compute full residuals/fitted values
-    if (e$e.is.res)
+    if (e[["e.is.res"]])
       draw <- add(draw, bquote(mv_update(p[["e_"]], plus=FALSE, p[[.(name_X)]], p[[.(name)]])))
     else
       draw <- add(draw, bquote(mv_update(p[["e_"]], plus=TRUE, p[[.(name_X)]], p[[.(name)]])))
@@ -357,9 +353,7 @@ mec <- function(formula = ~ 1, sparse=NULL, X=NULL, V=NULL,
       p[[name_X]] <- X
       p
     }
-  }  # END if (!e$prior.only)
-
-  rm(R, r, S, s, lower, upper)
+  }  # END if (!prior.only)
 
   environment()
 }
