@@ -1,9 +1,9 @@
 # mc: model component
 # update.XX: whether XX is updated in each draw
-# add.outer.R: whether outer product of constraints should be added (for positive definiteness in blocked components)
+# control: sampler options
 # this function is called for its side effect of assigning a MVN sampler object
 #   to environment mc, as well as an update method and possibly mat_sum and kron_prod methods
-sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
+sparse_template <- function(mc, update.XX=FALSE, control=NULL) {
 
   if (mc[["type"]] == "block") {
     Q <- mc[["QT"]]
@@ -12,23 +12,26 @@ sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
     # build a Kronecker product template
     Qv <- mc$generate_Qv()
     if (mc[["gl"]]) {
-      QA <- mc$glp$QA.ext
-    } else if (mc$strucA$update.Q) {
+      QA <- mc$glp[["QA.ext"]]
+    } else if (mc$strucA[["update.Q"]]) {
       QA <- mc$strucA$update_Q(mc[["QA"]], runif(1L, 0.25, 0.75))
-    } else if (is.null(mc$priorA)) {
+    } else if (is.null(mc[["priorA"]])) {
       QA <- if (is.null(mc[["AR1.inferred"]])) mc[["QA"]] else mc$QA.template$QA0.5
     } else {
       if (is.null(mc[["AR1.inferred"]]))
-        QA <- crossprod_sym(mc$DA, runif(mc$lD, 0.75, 1.25))
+        QA <- crossprod_sym(mc[["DA"]], runif(mc[["lD"]], 0.75, 1.25))
       else
-        QA <- crossprod_sym(mc$DA.template$DA0.5, runif(mc$lD, 0.75, 1.25))
+        QA <- crossprod_sym(mc$DA.template[["DA0.5"]], runif(mc[["lD"]], 0.75, 1.25))
     }
 
     kron_prod <- build_kron(QA, Qv, q2=mc[["q0"]],
-      M1.fixed = is.null(mc[["priorA"]]) && !mc$strucA$update.Q && is.null(mc[["AR1.inferred"]])
+      M1.fixed = is.null(mc[["priorA"]]) && !mc$strucA[["update.Q"]] && is.null(mc[["AR1.inferred"]])
     )
     Q <- kron_prod(QA, Qv)
-    if (mc[["in_block"]]) assign("Q", Q, envir=mc)  # only for single-time use in create_mc_block
+    if (mc[["in_block"]]) {
+      if (is.matrix(Q)) Q <- as(as(Q, "CsparseMatrix"), "symmetricMatrix")
+      assign("Q", Q, envir=mc)  # only for single-time use in create_mc_block
+    }
     keep.kp <- TRUE
   }
 
@@ -41,7 +44,10 @@ sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
       R <- mc[["R"]]
     }
 
-    if (mc[["e"]]$control$expanded.cMVN.sampler) {
+    add.eps.I <- if (is.null(control)) FALSE else control[["add.eps.I"]]
+    add.outer.R <- if (add.eps.I || is.null(control)) FALSE else control[["add.outer.R"]]
+    cMVN.sampler <- if (is.null(control)) FALSE else control[["cMVN.sampler"]]
+    if (cMVN.sampler) {
       if (update.XX) {
         mat_sum <- make_mat_sum(M1=XX, M2=Q, force.sparse=TRUE)
         XX_Q <- mat_sum(XX, Q)
@@ -53,7 +59,7 @@ sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
         mbs=mc[["mcs"]], X=mc[["X"]], Q=XX_Q,
         R=R, r=mc[["r"]],
         sampler=mc[["e"]],
-        name=mc[["name"]], chol.control=mc[["e"]]$control$chol.control
+        name=mc[["name"]], chol.control=mc[["e"]]$control[["chol.control"]]
       )
     } else {
       mc$MVNsampler <- NULL
@@ -64,7 +70,7 @@ sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
         if (add.outer.R && !is.null(R)) {
           # it should not always be necessary to use the full matrix R
           # leave out the largest of the individual model components' contribution:
-          cols <- i_apply(mc[["mcs"]], function(x) if (is.null(x[["R"]])) 0L else ncol(x[["R"]]))
+          cols <- i_apply(mc[["mcs"]], \(x) if (is.null(x[["R"]])) 0L else ncol(x[["R"]]))
           cols <- cols[cols != 0L]
           if (length(cols) > 1L) {
             inds <- c(0L, cumsum(cols))
@@ -73,6 +79,10 @@ sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
           }
         }
       }
+
+      # possibly affecting constraints: IGMRF factors, user-defined constraints, bym2, gl
+      constraints <- set_constraints(R=R, r=mc[["r"]], S=mc[["S"]], s=mc[["s"]])
+
       if (!add.outer.R || is.null(R)) {
         if (update.XX) {
           mat_sum <- make_mat_sum(M1=XX, M2=Q)
@@ -85,8 +95,8 @@ sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
           suppressWarnings(
             mc$MVNsampler <- create_TMVN_sampler(
               Q=XX_Q, update.Q=TRUE, name=mc[["name"]],
-              R=R, r=mc[["r"]], S=mc[["S"]], s=mc[["s"]],
-              chol.control=mc[["e"]]$control$chol.control
+              constraints=constraints,
+              chol.control=mc[["e"]]$control[["chol.control"]]
             )
           ),
           error = function(e) {
@@ -99,53 +109,60 @@ sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
             # TODO add tcrossprod(R0) where R0 is the minimal selection of columns s.t. XX + R0 R0' is positive definite
             #   i.e. restrict R0 to a selection of constraints that involve all non-observed levels
             #   and perhaps use weights to further reduce the condition number
-            if (is.null(R)) {
+            if (!add.eps.I && is.null(R)) {
               if (mc[["type"]] == "block") {
-                noninf <- b_apply(mc[["mcs"]], function(x) isFALSE(x$informative.prior))
+                noninf <- b_apply(mc[["mcs"]], \(x) isFALSE(x[["informative.prior"]]))
                 if (any(noninf))
-                  warn("error in setting up a multivariate normal sampler, possibly due to ",
-                       "a singular precision matrix; you may try to replace the non-informative prior ",
-                       "of model component(s) '", paste0(names(noninf)[noninf], collapse="', '"),
-                       "' by a (weakly) informative prior")
+                  message("Error in setting up a multivariate normal sampler, possibly due to ",
+                       "a singular precision matrix. You may try to replace the non-informative prior ",
+                       "of model component(s) '", paste0(names(mc[["mcs"]])[noninf], collapse="', '"),
+                       "' by a (weakly) informative prior. Alternatively set 'add.eps.I' to TRUE in create_sampler control argument.")
               }
               stop(e)
             } else {
-              if (class(XX_Q)[1L] == "dsCMatrix" && XX_Q@Dim[1L] >= 1000L) {
-                message("adding outer product of constraint matrix to the ",
-                        "precision matrix to improve its condition number; note that this may be ",
-                        "inefficient if the resulting matrix is much less sparse")
+              if (!add.eps.I && class(XX_Q)[1L] == "dsCMatrix" && XX_Q@Dim[1L] >= 500L) {
+                message("adding outer product of constraint matrix to the precision matrix ",
+                  "to improve its condition number\n",
+                  "note that this may be inefficient if the resulting matrix is much less sparse"
+                )
               }
             }
           }
         )
       }
-      if (is.null(mc$MVNsampler)) {  # 2nd attempt or add.outer.R
+      if (is.null(mc[["MVNsampler"]])) {  # 2nd attempt or add.outer.R
         if (is.null(R)) stop("singular precision matrix and no constraints")
         if (update.XX) {
-          mat_sum <- make_mat_sum(M0=tcrossprod(R0), M1=XX, M2=Q)
+          if (add.eps.I)
+            mat_sum <- make_mat_sum(M0=control[["eps"]] * CdiagU(ncol(XX)), M1=XX, M2=Q)
+          else
+            mat_sum <- make_mat_sum(M0=tcrossprod(R0), M1=XX, M2=Q)
           XX_Q <- mat_sum(XX, Q)
         } else {
-          mat_sum <- make_mat_sum(M0=economizeMatrix(XX + tcrossprod(R0), symmetric=TRUE), M1=Q)
+          if (add.eps.I)
+            mat_sum <- make_mat_sum(M0=economizeMatrix(XX + control[["eps"]] * CdiagU(ncol(XX)), symmetric=TRUE), M1=Q)
+          else
+            mat_sum <- make_mat_sum(M0=economizeMatrix(XX + tcrossprod(R0), symmetric=TRUE), M1=Q)
           XX_Q <- mat_sum(Q)
         }
         tryCatch(
           suppressWarnings(
             mc$MVNsampler <- create_TMVN_sampler(
               Q=XX_Q, update.Q=TRUE, name=mc[["name"]],
-              R=R, r=mc[["r"]], S=mc[["S"]], s=mc[["s"]],
-              chol.control=mc[["e"]]$control$chol.control
+              constraints=constraints,
+              chol.control=mc[["e"]]$control[["chol.control"]]
             )
           ),
           error = function(e) {
             if (mc[["type"]] == "block") {
-              noninf <- b_apply(mc[["mcs"]], function(x)
-                isFALSE(x$informative.prior) || (x[["type"]] == "gen" && x[["gl"]] && isFALSE(x$glp$informative.prior))
+              noninf <- b_apply(mc[["mcs"]], \(x)
+                isFALSE(x[["informative.prior"]]) || (x[["type"]] == "gen" && x[["gl"]] && isFALSE(x$glp$informative.prior))
               )
               if (any(noninf))
-                warn("error in setting up a multivariate normal sampler, possibly due to ",
-                     "a singular precision matrix; you may try to replace the non-informative prior ",
-                     "of model component(s) '", paste0(names(noninf)[noninf], collapse="', '"),
-                     "' by a (weakly) informative prior")
+                warn("Error in setting up a multivariate normal sampler, possibly due to ",
+                     "a singular precision matrix. You may try to replace the non-informative prior ",
+                     "of model component(s) '", paste0(names()[noninf], collapse="', '"),
+                     "' by a (weakly) informative prior.")
             }
             stop(e)
           }
@@ -157,7 +174,7 @@ sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
     if (mc[["type"]] == "gen" && mc[["unit_Q"]]) {  # unit QA, scalar Qv (with unit Q0), and no constraints
       if (is_unit_diag(XX)) {
         # TODO cleaner and more efficient handling of unit XX case (unit_Q + unit XX --> scalar update)
-        if (is_unit_ddi(XX.expanded)) XX.expanded <- expand_unit_ddi(XX)
+        XX.expanded <- if (is_unit_ddi(XX)) expand_unit_ddi(XX) else XX
         mc$update <- function(XX, QA, Qv, w) list(Q=XX.expanded, Imult=Qv*w)
         keep.kp <- keep.ms <- FALSE
       } else {
@@ -188,7 +205,7 @@ sparse_template <- function(mc, update.XX=FALSE, add.outer.R=FALSE) {
     if (keep.ms) mc$mat_sum <- mat_sum
   }
 
-  if (mc[["type"]] != "block" && mc$in_block) keep.kp <- TRUE
+  if (mc[["type"]] != "block" && mc[["in_block"]]) keep.kp <- TRUE
   if (keep.kp) mc$kron_prod <- kron_prod
 
 }
